--[===[

Pandoc filter which solves a twofold problem when you have several Markdown files containing different sections of the same text with links between the different sections and want to be able to generate either one big file with all the sections or separate files, and that in different output formats:

1.  If you generate one big file with the whole text you want internal
    targets like `#title-b` but when you generate separate files from
    each source file containing a major section you want external
    targets like `b.html`, `b.pdf` or even `b.html#b-title`.

2.  You probably want the file extensions in your external targets to be
    the one appropriate for each generated file format rather than the
    file extension of your source files.

Luckily both problems are easy to solve with a filter or two:

1.  Since Link elements take attributes in recent versions of Pandoc you
    can put the internal target in the target slot of your link elements
    and add the external target as a custom attribute, i.e. instead of
    `[here](b.html)` *or* `[here](#b-title)` you type
    `[here](#b-title){file=b.html}` and let a filter replace the
    internal target in the target slot with the external target in the
    attribute when generating multi-file output. (Theoretically you
    could do it the other way around, but its hard to come up with a
    good attribute name as short as `file` in that case! :-)

2.  Here you can have a filter replace the `.md` extension --- or even a
    dummy extension like `.foo` --- with the appropriate extension for
    each output format.

Ideally you want a configurable filter. Filters can be configured either
through document metadata --- which can also be set on the command line
with the `-M` option --- or with environment variables. I usually
provide for both so that I can have the general case configured in the
environment and then override it with metadata as needed.

So in this filter you have the following configuration variables:

| Metadata key    | Purpose                                 | Default |
|:----------------|:----------------------------------------|:--------|
| `file_urls`     | Replace internal targets with external? | false   |
| `file_url_attr` | The attribute to use for file urls.     | `file`  |
| `dummy_ext`     | The extension which should be replaced. | `md`    |
| `real_ext`      | The extension to replace it with.       | `html`  |

All the environment variable names are derived from the metadata keys by
capitalizing them and prepending the prefix `PANDOC_CROSSURLS_`, so
`PANDOC_CROSSURLS_FILE_URLS` and so on. Since the `file_urls` variable
is a boolean the environment variable value has to be converted to one:
the value `false` (case insensitive) counts as false, all other defined
values as true. Undefined/unset means that the metadata value wins.

So when generating a single file (with internal links) you just say

    pandoc --lua-filter pandoc-crossurls.lua -M file_urls=false -o combined.pdf *.md

and when generating a separate file you say

    pandoc --lua-filter pandoc-crossurls.lua -M file_urls -M real_ext=pdf -o a.pdf a.md

| This software is Copyright (c) 2019 by Benct Philip Jonsson.
| 
| This is free software, licensed under:
| 
|   The MIT (X11) License
| 
| The MIT License
| 
| Permission is hereby granted, free of charge, to any person
| obtaining a copy of this software and associated
| documentation files (the "Software"), to deal in the Software
| without restriction, including without limitation the rights to
| use, copy, modify, merge, publish, distribute, sublicense,
| and/or sell copies of the Software, and to permit persons to
| whom the Software is furnished to do so, subject to the
| following conditions:
| 
| The above copyright notice and this permission notice shall
| be included in all copies or substantial portions of the
| Software.
| 
| THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
| WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
| INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
| MERCHANTABILITY, FITNESS FOR A PARTICULAR
| PURPOSE AND NONINFRINGEMENT. IN NO EVENT
| SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
| LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
| LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
| TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
| CONNECTION WITH THE SOFTWARE OR THE USE OR
| OTHER DEALINGS IN THE SOFTWARE.

]===]

local utils = require"pandoc.utils"

local config = {
    file_urls     = false,
    file_url_attr = 'file',
    dummy_ext     = 'md',
    real_ext      = 'html',
}

local function stringify (x)
    if 'table' == type(x) and x.t then
        return utils.stringify(x)
    else
        return x
    end
end

local function get_config (meta)
    for key,default in pairs(config) do
        if nil ~= meta[key] then
            config[key] = stringify(meta[key])
        else
            local env = os.getenv("PANDOC_CROSSURLS_" .. key:upper())
            if env then 
                if 'boolean' == type(default) then
                    if 'false' == env:lower() then
                        env = false
                    else
                        env = true
                    end
                end
                config[key] = env
            end
        end
    end
    -- prepend dot to extensions as needed
    for _,key in ipairs({'dummy_ext', 'real_ext'}) do
        config[key] = tostring(config[key]):gsub('^%.?', '.')
    end
    -- convert dummy_ext to a pattern
    config.dummy_ext = '%' .. config.dummy_ext .. '$'
    return nil
end

local function do_link (elem)
    local return_value = nil
    if config.file_urls then
        if elem.attributes[config.file_url_attr] then
            elem.target = elem.attributes[config.file_url_attr]
        end
    end
    elem.target = tostring(elem.target):gsub(config.dummy_ext, config.real_ext)
    return elem
end

return {
    { Meta = get_config },
    { Link = do_link },
}
